@@ -1,13 +1,13 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	@(#) mhamain.pl 2.31 01/11/24 01:53:40
+##	$Id: mhamain.pl,v 2.74 2004/05/17 05:17:51 ehood Exp $
 ##  Author:
 ##      Earl Hood       mhonarc@mhonarc.org
 ##  Description:
 ##	Main library for MHonArc.
 ##---------------------------------------------------------------------------##
 ##    MHonArc -- Internet mail-to-HTML converter
-##    Copyright (C) 1995-2001	Earl Hood, mhonarc@mhonarc.org
+##    Copyright (C) 1995-2004	Earl Hood, mhonarc@mhonarc.org
 ##
 ##    This program is free software; you can redistribute it and/or modify
 ##    it under the terms of the GNU General Public License as published by
@@ -27,15 +27,38 @@
 
 package mhonarc;
 
-$VERSION = "2.5.2";
+require 5;
+
+$VERSION = '2.6.10';
 $VINFO =<<EndOfInfo;
   MHonArc v$VERSION (Perl $] $^O)
-  Copyright (C) 1995-2001  Earl Hood, mhonarc\@mhonarc.org
+  Copyright (C) 1995-2004  Earl Hood, mhonarc\@mhonarc.org
   MHonArc comes with ABSOLUTELY NO WARRANTY and MHonArc may be copied only
   under the terms of the GNU General Public License, which may be found in
   the MHonArc distribution.
 EndOfInfo
 
+###############################################################################
+BEGIN {
+    ## Check what system we are executing under
+    require 'osinit.pl';  &OSinit();
+
+    ## Check if running setuid/setgid
+    $TaintMode = 0;
+    if ($UNIX && (( $< != $> ) || ( $( != $) ))) {
+	## We do not support setuid since there are too many
+	## security problems to handle, and if we did, mhonarc
+	## would probably not be very useful.
+	die "ERROR: setuid/setgid execution not supported!\n";
+
+	#$TaintMode = 1;
+	#$ENV{'PATH'}  = '/bin:/usr/bin';
+	#$ENV{'SHELL'} = '/bin/sh'  if exists $ENV{'SHELL'};
+	#delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};
+    }
+}
+###############################################################################
+
 $CODE		= 0;
 $ERROR  	= "";
 @OrgARGV	= ();
@@ -49,7 +72,6 @@ my @_term_sigs	= qw(
     TERM TRAP USR1 USR2 VTALRM XCPU XFSZ
 );
 
-
 ###############################################################################
 ##	Public routines
 ###############################################################################
@@ -62,9 +84,6 @@ sub initialize {
     ##	Turn off buffered I/O to terminal
     my($curfh) = select(STDOUT);  $| = 1;  select($curfh);
 
-    ##	Check what system we are executing under
-    require 'osinit.pl';  &OSinit();
-
     ##	Require essential libraries
     require 'mhlock.pl';
     require 'mhopt.pl';
@@ -257,9 +276,9 @@ sub doit {
     }
 
     ## Get here, we are processing mail folders
-    my($index, $fields, $fh, $i);
+    my($index, $fields, $fh, $cur_msg_cnt);
 
-    $i = $NumOfMsgs;
+    $cur_msg_cnt = $NumOfMsgs;
     ##-------------------##
     ## Read mail folders ##
     ##-------------------##
@@ -277,14 +296,7 @@ sub doit {
 
 	if ($index) {
 	    $AddIndex{$index} = 1;
-	    $IndexNum{$index} = &getNewMsgNum();
-
-	    ## Read rest of message
-	    $Message{$index} = &read_mail_body(
-					$handle,
-					$index,
-				        $fields,
-					$NoMsgPgs);
+	    read_mail_body($handle, $index, $fields, $NoMsgPgs);
 	}
 
     ## Adding/converting mail{boxes,folders}
@@ -293,7 +305,7 @@ sub doit {
 	    unless $QUIET;
 	my($mbox, $mesgfile, @files);
 
-	foreach $mbox (@ARGV) {
+	MAILFOLDER: foreach $mbox (@ARGV) {
 
 	    ## MH mail folder (a directory)
 	    if (-d $mbox) {
@@ -306,11 +318,17 @@ sub doit {
 		@files = sort { $a <=> $b } grep(/$MHPATTERN/o,
 						 readdir(MAILDIR));
 		closedir(MAILDIR);
-		foreach (@files) {
+
+		local($_);
+		MHFILE: foreach (@files) {
 		    $mesgfile = "${mbox}${DIRSEP}${_}";
-		    if (!($fh = file_open($mesgfile))) {
-			warn "\nWarning: Unable to open message $mesgfile\n";
-			next;
+		    eval {
+			$fh = file_open($mesgfile);
+		    };
+		    if ($@) {
+			warn $@,
+			     qq/...Skipping "$mesgfile"\n/;
+			next MHFILE;
 		    }
 		    print STDOUT "."  unless $QUIET;
 		    ($index, $fields) = read_mail_header($fh);
@@ -318,12 +336,8 @@ sub doit {
 		    #  Process message if valid
 		    if ($index) {
 			if ($ADD && !$SLOW) { $AddIndex{$index} = 1; }
-			$IndexNum{$index} = &getNewMsgNum();
-			$Message{$index} = &read_mail_body(
-						$fh,
-						$index,
-					        $fields,
-						$NoMsgPgs);
+			read_mail_body($fh, $index, $fields, $NoMsgPgs);
+
 			#  Check if conserving memory
 			if ($SLOW && $DoArchive) {
 			    output_mail($index, 1, 1);
@@ -341,9 +355,15 @@ sub doit {
 	    } else {
 		if ($mbox eq "-") {
 		    $fh = $MhaStdin;
-		} elsif (!($fh = &file_open($mbox))) {
-		    warn "\nWarning: Unable to open $mbox\n";
-		    next;
+		} else {
+		    eval {
+			$fh = file_open($mbox);
+		    };
+		    if ($@) {
+			warn $@,
+			     qq/...Skipping "$mbox"\n/;
+			next MAILFOLDER;
+		    }
 		}
 
 		$MBOX = 1;  $MH = 0;
@@ -355,12 +375,8 @@ sub doit {
 
 		    if ($index) {
 			if ($ADD && !$SLOW) { $AddIndex{$index} = 1; }
-			$IndexNum{$index} = &getNewMsgNum();
-			$Message{$index} = read_mail_body(
-						$fh,
-						$index,
-						$fields,
-						$NoMsgPgs);
+			read_mail_body($fh, $index, $fields, $NoMsgPgs);
+
 			if ($SLOW && $DoArchive) {
 			    output_mail($index, 1, 1);
 			    $Update{$IndexNum{$index}} = 1;
@@ -387,11 +403,12 @@ sub doit {
     }
 
     ## Check if there are any new messages
-    if (!$EDITIDX && ($i == $NumOfMsgs)) {
+    if (!$EDITIDX && ($cur_msg_cnt > 0) &&
+	    !scalar(%AddIndex) && !scalar(%Update)) {
 	print STDOUT "No new messages\n"  unless $QUIET;
 	return 1;
     }
-    $NewMsgCnt = $NumOfMsgs - $i;
+    $NewMsgCnt = $NumOfMsgs - $cur_msg_cnt;
 
     ## Write pages
     &write_pages();
@@ -402,8 +419,9 @@ sub doit {
 ##	write_pages writes out all archive pages and db
 ##
 sub write_pages {
-    my($i, $key, $index, $tmp, $tmp2);
+    my($i, $j, $key, $index, $tmp, $tmp2);
     my(@array2);
+    my($mloc, $tloc);
 
     ## Remove old message if hit maximum size or expiration
     if (($MAXSIZE && ($NumOfMsgs > $MAXSIZE)) ||
@@ -421,7 +439,6 @@ sub write_pages {
 	&ign_signals();
 
 	## Expiration based upon time
-	my($mloc, $tloc);
 	foreach $index (sort_messages(0,0,0,0)) {
 	    last  unless
 		    ($MAXSIZE && ($NumOfMsgs > $MAXSIZE)) ||
@@ -531,6 +548,16 @@ sub write_pages {
 			if $i > 0;
 		    $Update{$IndexNum{$TListOrder[$i+1]}} = 1
 			if $i < $#TListOrder;
+
+		    $tloc = $Index2TLoc{$index};
+		    for ($j=2; $j <= $TSliceNBefore; ++$j) {
+			$Update{$IndexNum{$TListOrder[$tloc-$j]}} = 1
+			    if $tloc-$j >= 0;
+		    }
+		    for ($j=2; $j <= $TSliceNAfter; ++$j) {
+			$Update{$IndexNum{$TListOrder[$tloc+$j]}} = 1
+			    if $tloc-$j >= $#TListOrder;
+		    }
 		}
 		$i++;
 	    }
@@ -603,7 +630,7 @@ sub write_pages {
 	}
 	    
 	## read resource file and print index
-	if (read_fmt_file($rcfile)) {
+	if (read_resource_file($rcfile)) {
 	    if ($THREAD) {
 		@TListOrder = ();
 		write_thread_index();
@@ -690,12 +717,14 @@ sub write_mail {
 ##
 sub read_mail_header {
     my $handle = shift;
-    my($index, $date, $tmp);
+    my($date, $tmp, $i, $field, $value);
     my($from, $sub, $msgid, $ctype);
     local($_);
 
-    my @refs = ();
-    my @array = ();
+    my $index  = undef;
+    my $msgnum = undef;
+    my @refs   = ();
+    my @array  = ();
     my($fields, $header) = readmail::MAILread_file_header($handle);
 
     ##---------------------------##
@@ -745,24 +774,37 @@ sub read_mail_header {
     }
 
     ## Return if message already exists in archive
-    if ($msgid && defined($MsgId{$msgid})) {
-	return undef;
+    if ($msgid && defined($index = $MsgId{$msgid})) {
+	if ($Reconvert) {
+	    $msgnum = $IndexNum{$index};
+	    delmsg($index);
+	    $index = undef;
+	} else {
+	    return undef;
+	}
     }
 
     ##----------##
     ## Get date ##
     ##----------##
     $date = "";
-    foreach (@DateFields) {
-	next  unless defined($fields->{$_});
+    foreach (@_DateFields) {
+	($field, $i) = @{$_}[0,1];
+	next  unless defined($fields->{$field}) &&
+		     defined($value = $fields->{$field}[$i]);
 
 	## Treat received field specially
-	if ($_ eq 'received') {
-	    @array = split(/;/, $fields->{$_}[0]);
+	if ($field eq 'received') {
+	    @array = split(/;/, $value);
+#	    if ((scalar(@array) <= 1) || (scalar(@array) > 2)) {
+#		warn qq/\nWarning: Received header field looks improper:\n/,
+#		       qq/         Received: $value\n/,
+#		       qq/         Message-Id: <$msgid>\n/;
+#	    }
 	    $date = pop @array;
 	## Any other field should just be a date
 	} else {
-	    $date = $fields->{$_}[0];
+	    $date = $value;
 	}
 	$date =~ s/^\s+//;  $date =~ s/\s+$//;
 
@@ -838,7 +880,7 @@ sub read_mail_header {
     }
 
     ## Insure uniqueness of index
-    $index .= $X . sprintf("%d",$LastMsgNum+1);
+    $index .= $X . sprintf('%d',(defined($msgnum)?$msgnum:($LastMsgNum+1)));
 
     ## Set mhonarc fields.  Note how values are NOT arrays.
     $fields->{'x-mha-index'} = $index;
@@ -862,9 +904,28 @@ sub read_mail_header {
 	$NewMsgId{$msgid} = $index;	# Track new message-ids
 	$Index2MsgId{$index} = $msgid;
     }
+    if (defined($msgnum)) {
+	$IndexNum{$index} = $msgnum;
+	++$NumOfMsgs; # Counteract decrement by delmsg
+    } else {
+	$IndexNum{$index} = getNewMsgNum();
+    }
 
     $Refs{$index} = [ @refs ]  if (@refs);
 
+    ## Grab any extra fields to store
+    foreach $field (@ExtraHFields) {
+	next  unless $fields->{$field};
+	if (!defined($tmp = $ExtraHFields{$index})) {
+	    $tmp = $ExtraHFields{$index} = { };
+	}
+	if ($HFieldsAddr{$field}) {
+	    $tmp->{$field} = join(', ', @{$fields->{$field}});
+	} else {
+	    $tmp->{$field} = join(' ', @{$fields->{$field}});
+	}
+    }
+
     ($index, $fields);
 }
 
@@ -881,11 +942,6 @@ sub read_mail_body {
     my(@files);
     local($_);
 
-    ## Define "globals" for use by filters
-    ##	NOTE: This stuff can be handled better, and will be done
-    ##	      when/if I get around to rewriting mhonarc in (OO) Perl 5.
-    $MHAmsgnum = &fmt_msgnum($IndexNum{$index}) unless $skip;
-
     ## Slurp up message body
     ##	UUCP mailbox
     if ($MBOX) {
@@ -916,8 +972,21 @@ sub read_mail_body {
 	$data = <$handle>;
     }
 
-    ## Filter data
     return ''  if $skip;
+
+    ## Invoke callback if defined
+    if (defined($CBRawMessageBodyRead) && defined(&$CBRawMessageBodyRead)) {
+	&$CBRawMessageBodyRead($fields, \$data);
+    }
+
+    ## Define "globals" for use by filters
+    ##	NOTE: This stuff can be handled better, and will be done
+    ##	      when/if I get around to rewriting mhonarc in (OO) Perl 5.
+    $MHAindex  = $index;
+    $MHAmsgnum = &fmt_msgnum($IndexNum{$index});
+    $MHAmsgid  = $Index2MsgId{$index};
+
+    ## Filter data
     ($ret, @files) = &readmail::MAILread_body($fields, \$data);
     $ret = ''     unless defined $ret;
     @files = ( )  unless @files;
@@ -925,19 +994,20 @@ sub read_mail_body {
     ## Invoke callback if defined
     if (defined($CBMessageBodyRead) && defined(&$CBMessageBodyRead)) {
 	&$CBMessageBodyRead($fields, \$ret, \@files);
+	$Message{$index} = $ret;
+    } else {
+	$Message{$index} = $ret;
     }
 
-    if (!defined($ret) || $ret eq "") {
-	$ret = join('',
-		    "<dl>\n",
-		    "<dt><strong>Warning</strong></dt>\n",
-		    "<dd>Unable to process data: \n",
-		    "<tt>",
-		    htmlize($fields->{'content-type'}[0] || 'text/plain'),
-		    "</tt>\n",
-		    "</dd>\n",
-		    "</dl>\n"
-		    );
+    if (!defined($ret) || $ret eq '') {
+	warn qq/\n/,
+	     qq/Warning: Empty body data generated:\n/,
+	     qq/         Message-Id: $MHAmsgid\n/,
+	     qq/         Message Number: $MHAmsgnum\n/,
+	     qq/         Content-Type/,
+			 ($fields->{'content-type'}[0] || 'text/plain'),
+			 qq/\n/;
+	$ret = '';
     }
     if (@files) {
 	$Derived{$index} = [ @files ];
@@ -951,26 +1021,50 @@ sub read_mail_body {
 ##	    $force	=> flag if mail is written and not editted, regardless
 ##	    $nocustom	=> ignore sections with user customization
 ##
+##	This function returns ($msgnum, $filename) if everything went
+##	okay, but no calls to this routine check the return values.
+##
 sub output_mail {
     my($index, $force, $nocustom) = @_;
     my($msgi, $tmp, $tmp2, $template, @array2);
-    my($msghandle, $msginfh, $drvfh);
+    my($msghandle, $msginfh);
+
+    my $msgnum	     = $IndexNum{$index};
+    if (!$SINGLE && !defined($msgnum)) {
+      # Something bad must have happened to message, so we just
+      # quietly return.
+      return;
+    }
 
     my $adding	     = ($ADD && !$force && !$SINGLE);
-    my $i_p0 	     = fmt_msgnum($IndexNum{$index});
-    my $filename     = msgnum_filename($IndexNum{$index});
+    my $i_p0 	     = fmt_msgnum($msgnum);
+    my $filename     = msgnum_filename($msgnum);
     my $filepathname = join($DIRSEP, $OUTDIR, $filename);
-    my $tmppathname  = join($DIRSEP, $OUTDIR, "msgtmp.$$");
+    my $tmppathname;
 
     if ($adding) {
-	return ($i_p0, $filename)  unless $Update{$IndexNum{$index}};
+	return ($i_p0, $filename)  unless $Update{$msgnum};
 	#&file_rename($filepathname, $tmppathname);
-	$msginfh = file_open($filepathname);
+	eval {
+	  $msginfh = file_open($filepathname);
+	};
+	if ($@) {
+	  # Something is screwed up with archive: We try to delete
+	  # message from database since message file appears to have
+	  # disappeared
+	  warn $@,
+	       qq/...Will attempt to remove message and continue on\n/;
+	  delmsg($index);
+
+	  # Nothing else to do, so return.
+	  return;
+	}
     }
     if ($SINGLE) {
 	$msghandle = \*STDOUT;
     } else {
-	$msghandle = file_create($tmppathname, $GzipFiles);
+	($msghandle, $tmppathname) =
+	    file_temp('tmsg'.$i_p0.'_XXXXXXXXXX', $OUTDIR);
     }
 
     ## Output HTML header
@@ -982,8 +1076,11 @@ sub output_mail {
     if (!$nocustom) {
 	#&defineIndex2MsgId();
 
-	($template = $SSMARKUP) =~ s/$VarExp/&replace_li_var($1,$index)/geo;
-	print $msghandle $template;
+	$template = ($MSGPGSSMARKUP ne '') ? $MSGPGSSMARKUP : $SSMARKUP;
+	if ($template ne '') {
+	    $template =~ s/$VarExp/&replace_li_var($1,$index)/geo;
+	    print $msghandle $template;
+	}
 
 	# Output comments -- more informative, but can be used for
 	#		     error recovering.
@@ -1046,7 +1143,7 @@ sub output_mail {
 	while (<$msginfh>) {
 	    # check if subject header delimited
 	    if (/<!--X-Subject-Header-Begin/) {
-		$tmp2 =~ s%($AddrExp)%&link_refmsgid($1,1)%geo;
+		$tmp2 =~ s/($HAddrExp)/&link_refmsgid($1,1)/geo;
 		print $msghandle $tmp2;
 		$tmp2 = "";
 
@@ -1062,7 +1159,7 @@ sub output_mail {
 	    }
 	    # check if head/body separator delimited
 	    if (/<!--X-Head-Body-Sep-Begin/) {
-		$tmp2 =~ s%($AddrExp)%&link_refmsgid($1,1)%geo;
+		$tmp2 =~ s/($HAddrExp)/&link_refmsgid($1,1)/geo;
 		print $msghandle $tmp2;
 		$tmp2 = "";
 
@@ -1080,7 +1177,7 @@ sub output_mail {
 	    $tmp2 .= $_;
 	    last  if /<!--X-MsgBody-End/;
 	}
-	$tmp2 =~ s%($AddrExp)%&link_refmsgid($1,1)%geo;
+	$tmp2 =~ s/($HAddrExp)/&link_refmsgid($1,1)/geo;
 	print $msghandle $tmp2;
 
     } else {
@@ -1091,8 +1188,8 @@ sub output_mail {
 	print $msghandle $template;
 	print $msghandle "<!--X-Subject-Header-End-->\n";
 
-	$MsgHead{$index} =~ s%($AddrExp)%&link_refmsgid($1)%geo;
-	$Message{$index} =~ s%($AddrExp)%&link_refmsgid($1)%geo;
+	$MsgHead{$index} =~ s/($HAddrExp)/&link_refmsgid($1)/geo;
+	$Message{$index} =~ s/($HAddrExp)/&link_refmsgid($1)/geo;
 
 	print $msghandle "<!--X-Head-of-Message-->\n";
 	print $msghandle $MsgHead{$index};
@@ -1140,9 +1237,10 @@ sub output_mail {
     if (!$nocustom && $DoFolRefs && defined($Refs{$index})) {
 	$tmp2 = 0;	# flag for when first ref printed
 	if (scalar(@{$Refs{$index}})) {
-	    foreach (@{$Refs{$index}}) {
-		next  unless defined($MsgId{$_});
-		next  unless defined($IndexNum{$MsgId{$_}});
+	    my($ref_msgid, $ref_index, $ref_num);
+	    foreach $ref_msgid (@{$Refs{$index}}) {
+		next  unless defined($ref_index = $MsgId{$ref_msgid});
+		next  unless defined($ref_num = $IndexNum{$ref_index});
 		if (!$tmp2) {
 		    ($template = $REFSBEGIN) =~
 			s/$VarExp/&replace_li_var($1,$index)/geo;
@@ -1150,7 +1248,7 @@ sub output_mail {
 		    $tmp2 = 1;
 		}
 		($template = $REFSLITXT) =~
-		    s/$VarExp/&replace_li_var($1,$MsgId{$_})/geo;
+		    s/$VarExp/&replace_li_var($1,$ref_index)/geo;
 		print $msghandle $template;
 	    }
 
@@ -1197,24 +1295,27 @@ sub output_mail {
 	close($msginfh);
 	#&file_remove($tmppathname);
     }
-    file_rename($tmppathname, $filepathname)  unless $SINGLE;
+    if (!$SINGLE) {
+	file_gzip($tmppathname)  if $GzipFiles;
+	file_chmod(file_rename($tmppathname, $filepathname));
+    }
 
     ## Create user defined files
+    my($drvfh);
     foreach (keys %UDerivedFile) {
 	($tmp = $_) =~ s/$VarExp/&replace_li_var($1,$index)/geo;
-	$tmp2 = join($DIRSEP, $OUTDIR, $tmp);
-	if ($drvfh = file_create($tmp2, $GzipFiles)) {
-	    ($template = $UDerivedFile{$_}) =~
-		s/$VarExp/&replace_li_var($1,$index)/geo;
-	    print $drvfh $template;
-	    close($drvfh);
-	    if (defined($Derived{$index})) {
-		push(@{$Derived{$index}}, $tmp);
-	    } else {
-		$Derived{$index} = [ $tmp ];
-	    }
+	($drvfh, $tmppathname) = file_temp('drvXXXXXXXXXX', $OUTDIR);
+	($template = $UDerivedFile{$_}) =~
+	    s/$VarExp/&replace_li_var($1,$index)/geo;
+	print $drvfh $template;
+	close($drvfh);
+	file_gzip($tmppathname)  if $GzipFiles;
+	file_chmod(file_rename($tmppathname, join($DIRSEP, $OUTDIR, $tmp)));
+
+	if (defined($Derived{$index})) {
+	    push(@{$Derived{$index}}, $tmp);
 	} else {
-	    warn "Warning: Unable to create $tmp2\n";
+	    $Derived{$index} = [ $tmp ];
 	}
     }
     if (defined($Derived{$index})) {
@@ -1225,11 +1326,20 @@ sub output_mail {
     if ($MODTIME && !$SINGLE) {
 	eval {
 	    $tmp = get_time_from_index($index);
-	    @array2 = @{$Derived{$index}};
-	    grep($_ = $OUTDIR . $DIRSEP . $_, @array2);
+	    if (defined($Derived{$index})) {
+	      @array2 = @{$Derived{$index}};
+	      grep($_ = $OUTDIR . $DIRSEP . $_, @array2);
+	    } else {
+	      @array2 = ( );
+	    }
 	    unshift(@array2, $filepathname);
 	    file_utime($tmp, $tmp, @array2);
 	};
+	if ($@) {
+	    warn qq/\nWarning: Your platform does not support setting file/,
+		   qq/         modification times\n/;
+	    $MODTIME = 0;
+	}
     }
 
     ($i_p0, $filename);
@@ -1243,7 +1353,7 @@ sub output_mail {
 ##	delmsg delets a message from the archive.
 ##
 sub delmsg {
-    my($key) = @_;
+    my $key = shift;
     my($pathname);
 
     #&defineIndex2MsgId();
@@ -1275,17 +1385,17 @@ sub delmsg {
 ##	Routine to convert a msgid to an anchor
 ##
 sub link_refmsgid {
-    my($refmsgid, $onlynew) = @_;
+    my $refmsgid = dehtmlize(shift);
+    my $onlynew  = shift;
 
     if (defined($MsgId{$refmsgid}) &&
-	defined($IndexNum{$MsgId{$refmsgid}}) &&
-	(!$onlynew || $NewMsgId{$refmsgid})) {
-	local($lreftmpl) = $MSGIDLINK;
+	    defined($IndexNum{$MsgId{$refmsgid}}) &&
+	    (!$onlynew || $NewMsgId{$refmsgid})) {
+	my($lreftmpl) = $MSGIDLINK;
 	$lreftmpl =~ s/$VarExp/&replace_li_var($1,$MsgId{$refmsgid})/geo;
-	$lreftmpl;
-    } else {
-	$refmsgid;
+	return $lreftmpl;
     }
+    htmlize($refmsgid);
 }
 
 ##---------------------------------------------------------------------------
